import axios from "axios";
import moxios from "moxios";
import {deepEqual, equal, notEqual} from "assert";
import sinon from "sinon";
import AlarmService from "../../../../src/domain/services/AlarmService";
import {
  ALARMSERVICE_CLEAR_CLIENT, ALARMSERVICE_CLEAR_INPUT, ALARMSERVICE_CLEAR_OUTPUT,
  ALARMSERVICE_CONFIRM_CLIENT,
  ALARMSERVICE_CONFIRM_INPUT,
  ALARMSERVICE_CONFIRM_OUTPUT,
  ALARMSERVICE_GETALL_INPUT,
  ALARMSERVICE_GETALL_OUTPUT
} from "../../../mock/deviceState/AlarmServiceMock";


describe('AlarmService_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('AlarmService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      AlarmService.getAllAlarmCollection();
      AlarmService.batchAlarmConfirm(ALARMSERVICE_CONFIRM_CLIENT);
      AlarmService.batchAlarmClear(ALARMSERVICE_CLEAR_CLIENT);
      //AlarmService.updateDeviceAndTopologyAndServerStatus();

      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 3);
        done()
      })
    });

    // 模拟响应
    it('should getAllAlarmCollection', function (done) {
      AlarmService.getAllAlarmCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: ALARMSERVICE_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, ALARMSERVICE_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should batchAlarmConfirm', function (done) {
      AlarmService.batchAlarmConfirm(ALARMSERVICE_CONFIRM_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: ALARMSERVICE_CONFIRM_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, ALARMSERVICE_CONFIRM_OUTPUT);
          done()
        })
      })
    });

    it('should batchAlarmClear', function (done) {
      AlarmService.batchAlarmClear(ALARMSERVICE_CLEAR_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: ALARMSERVICE_CLEAR_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, ALARMSERVICE_CLEAR_OUTPUT);
          done()
        })
      })
    });




  })
});
