import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import CDPService from '@/domain/services/Business/CDPService';
import {CDP_ADD_CLIENT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_DELETE_CLIENT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_GETALL_INPUT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_GETALL_OUTPUT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_ADD_INPUT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_ADD_OUTPUT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_DELETE_INPUT} from "../../../../mock/Business/CDPServiceMock";
import {CDP_DELETE_OUTPUT} from "../../../../mock/Business/CDPServiceMock";

describe('moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('CDPService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      CDPService.getAllCDPCollection();
      CDPService.addCDP(CDP_ADD_CLIENT);
      CDPService.deleteCDP(CDP_DELETE_CLIENT);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 3);
        done()
      })
    });

    // 模拟响应
    it('should getAllCDPCollection', function (done) {
      CDPService.getAllCDPCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CDP_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, CDP_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should addCDP', function (done) {
      CDPService.addCDP(CDP_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CDP_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          // console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, CDP_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should deleteCDP', function (done) {
      CDPService.deleteCDP(CDP_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CDP_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          // console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, CDP_DELETE_OUTPUT);
          done()
        })
      })
    });
  })
});
