import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import CabinetService from '@/domain/services/CabinetService'

import {CABINET_GETALL_INPUT, CABINET_GETALL_OUTPUT,
  CABINET_ADD_CLIENT, CABINET_ADD_INPUT, CABINET_ADD_OUTPUT,
  CABINET_DELETE_CLIENT, CABINET_DELETE_INPUT, CABINET_DELETE_OUTPUT,
  CABINET_MODDIFY_CLIENT, CABINET_MODDIFY_INPUT, CABINET_MODDIFY_OUTPUT,
  CABINET_QUERY_CLIENT, CABINET_QUERY_INPUT, CABINET_QUERY_OUTPUT,
  CABINET_DOWNLOAD_CLIENT, CABINET_UPLOAD_CLIENT
} from "../../../mock/deviceState/CabinetServiceMock";



describe('Cabinet_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('CabinetService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      CabinetService.getAllCabinetCollection();
      CabinetService.addCabinet(CABINET_ADD_CLIENT);
      CabinetService.deleteCabinets(CABINET_DELETE_CLIENT);
      CabinetService.modifyCabinet(CABINET_MODDIFY_CLIENT);
      CabinetService.conditionQueryCabinet(CABINET_QUERY_CLIENT);
      CabinetService.downloadCabinet(CABINET_DOWNLOAD_CLIENT);
      CabinetService.uploadCabinet(CABINET_UPLOAD_CLIENT);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 7);
        done()
      })
    });

    // 模拟响应
    it('should getAllCabinetCollection', function (done) {
      CabinetService.getAllCabinetCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CABINET_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          console.log(response);
          deepEqual(response, CABINET_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should addCabinet', function (done) {
      CabinetService.addCabinet(CABINET_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CABINET_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, CABINET_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should deleteLocations', function (done) {
      CabinetService.deleteCabinets(CABINET_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CABINET_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response,CABINET_DELETE_OUTPUT);
          done()
        })
      })
    });

    it('should modifyCabinet', function (done) {
      CabinetService.modifyCabinet(CABINET_MODDIFY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CABINET_MODDIFY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, CABINET_MODDIFY_OUTPUT);
          done()
        })
      })
    });

    it('should conditionQueryCabinet', function (done) {
      CabinetService.conditionQueryCabinet(CABINET_QUERY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: CABINET_QUERY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, CABINET_QUERY_OUTPUT);
          done()
        })
      })
    });


  })
});


