import axios from "axios";
import moxios from "moxios";
import {deepEqual, equal, notEqual} from "assert";
import sinon from "sinon";
import DeviceStateService from "../../../../src/domain/services/DeviceStateService.js";
import {
  GET_URL_INPUT, GET_URL_OUTPUT,
  GETALL_DEVICESTATE_INPUT,
  GETALL_DEVICESTATE_OUTPUT
} from "../../../mock/deviceState/DeviceStateServiceMock";



describe('DeviceStateService_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('DeviceStateService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      DeviceStateService.getAllDeviceState_TRAD();
      DeviceStateService.getAllDeviceState_SDNC();
      DeviceStateService.getWebSocketRequestURL();


      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 14);
        done()
      })
    });

    // 模拟响应
    /*it('should getAllDeviceState', function (done) {
      DeviceStateService.getAllDeviceState().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: GETALL_DEVICESTATE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, ALARMSERVICE_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should getWebSocketRequestURL', function (done) {
      let url=DeviceStateService.getWebSocketRequestURL();
      /!*moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: GET_URL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, GET_URL_OUTPUT);
          done()
        })
      })*!/
      deepEqual(url, GET_URL_OUTPUT);
    });*/

  })
});
