import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import LinkService from '@/domain/services/LinkService'
import {
  LINK_ADD_CLIENT, LINK_ADD_INPUT, LINK_ADD_OUTPUT,
  LINK_DELETE_CLIENT, LINK_DELETE_INPUT, LINK_DELETE_OUTPUT, LINK_GETALL_INPUT, LINK_GETALL_OUTPUT,
  LINK_MODDIFY_CLIENT, LINK_MODDIFY_INPUT, LINK_MODDIFY_OUTPUT,
  LINK_QUERY_CLIENT, LINK_QUERY_INPUT, LINK_QUERY_OUTPUT
} from "../../../mock/deviceState/LinkServiceMock";


describe('Link_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('LinkService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      LinkService.getAllLinkCollection();
      LinkService.addLink(LINK_ADD_CLIENT);
      LinkService.batchLinkDelete(LINK_DELETE_CLIENT);
      LinkService.modifyLink(LINK_MODDIFY_CLIENT);
      LinkService.conditionQueryLink(LINK_QUERY_CLIENT);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 5);
        done()
      })
    });

    // 模拟响应
    it('should getAllLinkCollection', function (done) {
      LinkService.getAllLinkCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: LINK_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, LINK_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should addLink', function (done) {
      LinkService.addLink(LINK_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: LINK_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, LINK_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should deleteLink', function (done) {
      LinkService.batchLinkDelete(LINK_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: LINK_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, LINK_DELETE_OUTPUT);
          done()
        })
      })
    });

    it('should modifyLink', function (done) {
      LinkService.modifyLink(LINK_MODDIFY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: LINK_MODDIFY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, LINK_MODDIFY_OUTPUT);
          done()
        })
      })
    });

    it('should conditionQueryLink', function (done) {
      LinkService.conditionQueryLink(LINK_QUERY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: LINK_QUERY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, LINK_QUERY_OUTPUT);
          done()
        })
      })
    });


  })
});
