import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import PortListService from '@/domain/services/PortListService'
import {
  PORT_ADD_CLIENT, PORT_ADD_INPUT, PORT_ADD_OUTPUT, PORT_ALL_INPUT, PORT_ALL_OUTPUT,
  PORT_DELETE_CLIENT, PORT_DELETE_INPUT, PORT_DELETE_OUTPUT, PORT_MODI_CLIENT, PORT_MODI_INPUT, PORT_MODI_OUTPUT,
  PORT_QUERY_CLIENT,
  PORT_QUERY_INPUT,
  PORT_QUERY_OUTPUT
} from "../../../mock/deviceState/PortListServiceMock";



describe('PortList_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('PortListService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      PortListService.conditionQueryPortList(PORT_QUERY_CLIENT);
      PortListService.batchDeletePortList(PORT_DELETE_CLIENT);
      PortListService.getAllPortList();
      PortListService.addPort(PORT_ADD_CLIENT);
      PortListService.modifyPort(PORT_MODI_CLIENT);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 5);
        done()
      })
    });

    // 模拟响应
    it('should conditionQueryPortList', function (done) {
      PortListService.conditionQueryPortList(PORT_QUERY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: PORT_QUERY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, PORT_QUERY_OUTPUT);
          done()
        })
      })
    });

    it('should batchDeletePortList', function (done) {
      PortListService.batchDeletePortList(PORT_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: PORT_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, PORT_DELETE_OUTPUT);
          done()
        })
      })
    });

    it('should getAllPortList', function (done) {
      PortListService.getAllPortList().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: PORT_ALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, PORT_ALL_OUTPUT);
          done()
        })
      })
    });

    it('should addPort', function (done) {
      PortListService.addPort(PORT_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: PORT_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, PORT_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should modifyPort', function (done) {
      PortListService.modifyPort(PORT_MODI_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: PORT_MODI_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, PORT_MODI_OUTPUT);
          done()
        })
      })
    });



  })
});
