import axios from "axios";
import moxios from "moxios";
import {deepEqual, equal, notEqual} from "assert";
import sinon from "sinon";
import AlarmFilterConfigService from "../../../../src/domain/services/AlarmFilterConfigService";
import {
  ALARMFILTER_ADD_CLIENT,
  ALARMFILTER_ADD_INPUT,
  ALARMFILTER_ADD_OUTPUT,
  ALARMFILTER_DELETE_CLIENT,
  ALARMFILTER_DELETE_INPUT, ALARMFILTER_DELETE_OUTPUT,
  ALARMFILTER_GETALL_INPUT,
  ALARMFILTER_GETALL_OUTPUT,
  ALARMFILTER_MODIFY_CLIENT,
  ALARMFILTER_MODIFY_INPUT,
  ALARMFILTER_MODIFY_OUTPUT
} from "../../../mock/deviceState/AlarmFilterConfigServiceMock";
import ServerService from "../../../../src/domain/services/ServerService";
import {
  SERVER_ADD_CLIENT,
  SERVER_ADD_INPUT,
  SERVER_ADD_OUTPUT,
  SERVER_DELETE_CLIENT,
  SERVER_DELETE_INPUT,
  SERVER_DELETE_OUTPUT,
  SERVER_GETALL_INPUT,
  SERVER_GETALL_OUTPUT,
  SERVER_MODDIFY_CLIENT,
  SERVER_MODDIFY_INPUT,
  SERVER_MODDIFY_OUTPUT,
  SERVER_QUERY_CLIENT,
  SERVER_QUERY_INPUT,
  SERVER_QUERY_OUTPUT,
  SERVERDEVICE_SDNC_CLIENT,
  SERVERDEVICE_SDNC_INPUT,
  SERVERDEVICE_SDNC_OUTPUT,
  SERVERDEVICENUM_SDNC_INPUT,
  SERVERDEVICENUM_SDNC_OUTPUT
} from "../../../mock/deviceState/ServerServiceMock";



describe('ServerService_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('ServerService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      ServerService.ServerService();
      ServerService.addServer(SERVER_ADD_CLIENT);
      ServerService.deleteServers(SERVER_DELETE_CLIENT);
      ServerService.modifyServer(SERVER_MODDIFY_CLIENT);
      ServerService.conditionQueryServer(SERVER_QUERY_CLIENT);
      ServerService.getDevicesUnderServer(SERVERDEVICE_SDNC_CLIENT);
      ServerService.getDeviceNumUnderServer();
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 7);
        done()
      })
    });

    // 模拟响应
    it('should ServerService', function (done) {
      ServerService.ServerService().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVER_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, SERVER_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should addServer', function (done) {
      ServerService.addServer(SERVER_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVER_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVER_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should deleteServers', function (done) {
      ServerService.deleteServers(SERVER_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVER_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVER_DELETE_OUTPUT);
          done()
        })
      })
    });

    it('should modifyServer', function (done) {
      ServerService.modifyServer(SERVER_MODDIFY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVER_MODDIFY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVER_MODDIFY_OUTPUT);
          done()
        })
      })
    });

    it('should conditionQueryServer', function (done) {
      ServerService.conditionQueryServer(SERVER_QUERY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVER_QUERY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVER_QUERY_OUTPUT);
          done()
        })
      })
    });

    it('should getDevicesUnderServer', function (done) {
      ServerService.getDevicesUnderServer(SERVERDEVICE_SDNC_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVERDEVICE_SDNC_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVERDEVICE_SDNC_OUTPUT);
          done()
        })
      })
    });

    it('should getDeviceNumUnderServer', function (done) {
      ServerService.getDeviceNumUnderServer().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SERVERDEVICENUM_SDNC_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SERVERDEVICENUM_SDNC_OUTPUT);
          done()
        })
      })
    });


  })
});
