import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import SubnetService from '@/domain/services/EdTemplateService'
import {
  SUBNET_ADD_CLIENT, SUBNET_ADD_INPUT, SUBNET_ADD_OUTPUT, SUBNET_DELETE_CLIENT, SUBNET_DELETE_INPUT,
  SUBNET_DELETE_OUTPUT,
  SUBNET_DOWNLOAD_CLIENT,
  SUBNET_GETALL_INPUT,
  SUBNET_GETALL_OUTPUT, SUBNET_MODDIFY_CLIENT, SUBNET_MODDIFY_INPUT, SUBNET_MODDIFY_OUTPUT, SUBNET_QUERY_CLIENT,
  SUBNET_QUERY_INPUT, SUBNET_QUERY_OUTPUT,
  SUBNET_UPLOAD_CLIENT
} from "../../../mock/deviceState/SubnetServiceMock";

describe('moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('EdTemplateService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      SubnetService.getAllSubnetCollection();
      SubnetService.addSubent(SUBNET_ADD_CLIENT);
      SubnetService.deleteSubnets(SUBNET_DELETE_CLIENT);
      SubnetService.modifySubnet(SUBNET_MODDIFY_CLIENT);
      SubnetService.conditionQuerySubnet(SUBNET_QUERY_CLIENT);
      SubnetService.downloadSubnet(SUBNET_DOWNLOAD_CLIENT);
      SubnetService.uploadSubnet(SUBNET_UPLOAD_CLIENT);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 7);
        done()
      })
    });

    // 模拟响应
    it('should getAllSubnetCollection', function (done) {
      SubnetService.getAllSubnetCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SUBNET_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, SUBNET_GETALL_OUTPUT);
          done()
        })
      })
    });

    it('should addSubent', function (done) {
      SubnetService.addSubent(SUBNET_ADD_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SUBNET_ADD_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SUBNET_ADD_OUTPUT);
          done()
        })
      })
    });

    it('should deleteSubnets', function (done) {
      SubnetService.deleteSubnets(SUBNET_DELETE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SUBNET_DELETE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SUBNET_DELETE_OUTPUT);
          done()
        })
      })
    });

    it('should modifySubnet', function (done) {
      SubnetService.modifySubnet(SUBNET_MODDIFY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SUBNET_MODDIFY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SUBNET_MODDIFY_OUTPUT);
          done()
        })
      })
    });

    it('should conditionQuerySubnet', function (done) {
      SubnetService.conditionQuerySubnet(SUBNET_QUERY_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: SUBNET_QUERY_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, SUBNET_QUERY_OUTPUT);
          done()
        })
      })
    });


  })
});
