import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import LinkService from '@/domain/services/LinkService'
import {
  TOPO_GETALL_INPUT,
  TOPO_GETALL_OUTPUT,
  TOPO_SAVE_CLIENT,
  TOPO_SAVE_INPUT, TOPO_SAVE_OUTPUT
} from "../../../mock/deviceState/TopologyServiceMock";
import TopologyService from "../../../../src/domain/services/TopologyService";


describe('Link_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('TopologyService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      LinkService.getAllLinkCollection();
      LinkService.addLink(TOPO_SAVE_CLIENT);

      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 2);
        done()
      })
    });

    // 模拟响应
    it('should getAllTopologyCollection', function (done) {
      TopologyService.getAllTopologyCollection().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: TOPO_GETALL_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, TOPO_GETALL_OUTPUT);
          done()
        })
      })
    });

    /*it('should saveTopologyCollection', function (done) {
      TopologyService.saveTopologyCollection(TOPO_SAVE_CLIENT).then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: TOPO_SAVE_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, TOPO_SAVE_OUTPUT);
          done()
        })
      })
    });*/




  })
});
