import { equal, notEqual, deepEqual,deepEql } from 'assert'
import axios from 'axios'
import moxios from 'moxios'
import sinon from 'sinon'
import UserGroupService from "../../../../src/domain/services/UserGroupService";
import {
  UG_ALLA_INPUT,
  UG_ALLA_OUTPUT,
  UG_ALLU_INPUT,
  UG_ALLU_OUTPUT
} from "../../../mock/deviceState/UserGroupServiceMock";

describe('UserGroupService_moxios', function () {
  it('should install', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    notEqual(axios.defaults.adapter, defaultAdapter)
    moxios.uninstall()
  })

  it('should uninstall', function () {
    let defaultAdapter = axios.defaults.adapter
    moxios.install()
    moxios.uninstall()
    equal(axios.defaults.adapter, defaultAdapter)
  })

  describe('UserGroupService (Rest)', function () {
    let onFulfilled
    let onRejected

    beforeEach(function () {
      moxios.install()
      // sinon.js 中 spy 主要用来监视函数的调用情况，
      // sinon 对待监视的函数进行 wrap 包装，
      // 因此可以通过它清楚的知道，该函数被调用过几次，传入什么参数返回什么结果，甚至是抛出的异常情况。
      onFulfilled = sinon.spy()
      onRejected = sinon.spy()
    })

    afterEach(function () {
      moxios.uninstall()
    })

    // 拦截请求
    it('should intercept requests', function (done) {
      UserGroupService.getAllAuthorities();
      UserGroupService.getAllUserGroups();
      //UserGroupService.createUserGroup(SUBNET_DELETE_CLIENT);
      //UserGroupService.deleteUserGroup(SUBNET_MODDIFY_CLIENT);
      //UserGroupService.updateUserGroup(SUBNET_QUERY_CLIENT);

      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        // console.log(moxios.requests);
        equal(moxios.requests.count(), 2);
        done()
      })
    });

    // 模拟响应
    it('should getAllAuthorities', function (done) {
      UserGroupService.getAllAuthorities().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: UG_ALLA_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          // console.log(response);
          deepEqual(response, UG_ALLA_OUTPUT);
          done()
        })
      })
    });

    it('should getAllUserGroups', function (done) {
      UserGroupService.getAllUserGroups().then(onFulfilled);
      moxios.wait(function () {
        let request = moxios.requests.mostRecent();
        request.respondWith({
          status: 200,
          response: UG_ALLU_INPUT
        }).then(function () {
          // console.log(onFulfilled.getCall(0));
          let response = onFulfilled.getCall(0).args[0];
          //console.log(response);
          equal(onFulfilled.called, true);
          // equal(response.status, 200);
          deepEqual(response, UG_ALLU_OUTPUT);
          done()
        })
      })
    });


  })
});
